.equ ESC,     0x1B

.equ Counter, 0x60
.equ k,       0x61
.equ NeuK,    0x62

.equ ASCIIPuffer,0x62

.org 0x100
  ; Interruptbearbeitung fr Timer 0 freigeben
  ; kein Interrupt fr serielle Schnittstelle
  mov 0xA8,#0x82  ; IE-Register

  ; Timer 0 als 8-Bit-Zhler mit Reload aus TH
  mov 0x89,#0x02  ; TMOD-Register

  ; Reload-Wert auf 0x00
  mov 0x8C,#0x00  ; TH0-Register
  mov 0x8A,#0x00  ; TL0-Register

  ; Erste LED einschalten
  mov P1,#0x01

  ; Counter und k initialiseren
  mov k,#0x7F
  mov Counter,#0x7F

  ; Timer 0 starten
  mov 0x88,#0x10  ; TCON-Register

Hauptschleife:

  mov r1,#ASCIIPuffer
  lcall ReadChars
  jnz Programmende

  mov r0,#ASCIIPuffer
  mov r1,#NeuK
  lcall ConvertToHex
  jz ok
  lcall Fehlermeldung
  ljmp Hauptschleife  
  ok:
  
  ; k=0x100-NeuK
  mov a,NeuK
  clr c
  subb a,#0x01
  clr c
  mov NeuK,a
  mov a,#0xFF
  subb a,NeuK
  mov k,a

ljmp Hauptschleife
  
Programmende:
  setb ES
  ljmp 0x918E


; =======================================================
; Interruptbehandlung fr Timer 0 Interrupt

.org 0x0B
  ljmp Timer0IRQ

.org 0x200
Timer0IRQ:
  djnz Counter,CountOnly
  mov Counter,k
  cpl P1.0
CountOnly:
  reti


; =======================================================
; Liet zwei Zeichen von der seriellen Schnittstelle und
; speichert sie in "ASCIIPuffer". Wird "Escape" gedrckt,
; so wird a=1 zurckgegeben, sonst a=0.

ReadChars:

  mov r0,#0x02
  mov r1,#ASCIIPuffer

LeseSchleife:

  Lesen: jnb RI,Lesen
  mov a,SBUF
  clr RI

  cjne a,#ESC,WeiterLesen
  mov a,#0x01
  ret

  WeiterLesen:
  mov @r1,a
  inc r1
  djnz r0,LeseSchleife
  mov a,#0x00
  ret


; =======================================================
; Wandelt ACSII-Zeichen aus @r0 in Hex-Wert um und speichert
; diesen in r1. Bei Fehler ist a ungleich 0, sonst gleich 0.

HalbConvertToHex:
  ; Abbruch, wenn ASCII-Wert kleiner 48 ist:
  mov a,@r0
  clr c
  subb a,#48  ; 48d = '0'
  jc HalbConvertFehler

  ; wenn ASCII-Wert kleiner 58, dann ist es eine Zahl.
  mov a,@r0
  clr c
  subb a,#58
  jc ZahlToHex

  ; Abbruch, wenn ASCII-Wert grer 70 ist:
  mov a,@r0
  clr c
  subb a,#71  ; 70d = 'F'
  jnc HalbConvertFehler

  ; wenn ASCII-Wert grer gleich 65, dann ist es ein Buchstabe.
  mov a,@r0
  clr c
  subb a,#65
  jnc ABCToHex

  sjmp HalbConvertFehler

ZahlToHex:
  mov a,@r0
  clr c
  subb a,#48
  mov @r1,a
  mov a,#0x00
  ret

ABCToHex:
  mov a,@r0
  clr c
  subb a,#55
  mov @r1,a
  mov a,#0x00
  ret

HalbConvertFehler:
  mov a,#0x01
  ret


; =======================================================
; Wandelt ACSII-Zeichen aus @r0 und @(r0+1) in Hex-Wert um
; und speichert diesen in @r1. Bei Fehler ist a ungleich 0,
; sonst gleich 0. Zustzlich wird r0 um 2 und r1 um 1 erhht.

ConvertToHex:
  ; r2-Register sichern
  mov B,r2
  push B

  lcall HalbConvertToHex ; ASCII @r0 -> Hex @r1 
  jnz ConvertEnde        ; wenn a ungleich 0: Fehler, Ende

  mov a,@r1 ; Ergebnis aus @r1 in das obere Halbbyte von r2
  mov B,#0x10
  mul ab      
  mov r2,a

  inc r0      

  lcall HalbConvertToHex ; ASCII @r0 -> Hex @r1 
  jnz ConvertEnde        ; wenn a ungleich 0: Fehler, Ende

  mov a,r2 ; Ergebnis aus r1 zu r2 addieren (unteres Halbbyte)
  add a,@r1
  mov @r1,a
  mov a,#0x00

  inc r0
  inc r1

  ConvertEnde:
  
  ; r2-Register wiederherstellen
  pop B
  mov r2,B

  ret


; =======================================================
; Gibt eine Fehlermeldung auf der seriellen Schnittstelle aus.

Fehlermeldung:
  mov dptr,#Meldungstext
  mov r2,#7

Meldungsschleife:
  mov a,#0x00  
  movc a,@a+dptr
  MeldungSchreiben: jnb TI,MeldungSchreiben
  clr TI
  mov SBUF,a
  inc dptr
  djnz r2,Meldungsschleife
  ret

  Meldungstext:
  .word 0x4665, 0x686C, 0x6572, 0x0A00 ; "Fehler"


.end